searchNodes=[{"ref":"Cldr.Message.html","title":"Cldr.Message","type":"module","doc":"Implements the ICU Message Format with functions to parse and interpolate messages."},{"ref":"Cldr.Message.html#format/3","title":"Cldr.Message.format/3","type":"function","doc":""},{"ref":"Cldr.Message.html#format!/3","title":"Cldr.Message.format!/3","type":"function","doc":""},{"ref":"Cldr.Message.html#format_list/3","title":"Cldr.Message.format_list/3","type":"function","doc":"Formats an ICU message into an iolist"},{"ref":"Cldr.Message.html#t:arguments/0","title":"Cldr.Message.arguments/0","type":"type","doc":""},{"ref":"Cldr.Message.html#t:message/0","title":"Cldr.Message.message/0","type":"type","doc":""},{"ref":"Cldr.Message.html#t:options/0","title":"Cldr.Message.options/0","type":"type","doc":""},{"ref":"Cldr.Message.Backend.html","title":"Cldr.Message.Backend","type":"module","doc":""},{"ref":"Cldr.Message.Backend.html#define_message_module/1","title":"Cldr.Message.Backend.define_message_module/1","type":"function","doc":""},{"ref":"Cldr.Message.Backend.html#expand_to_binary/2","title":"Cldr.Message.Backend.expand_to_binary/2","type":"function","doc":"Expands the given message in the given env, raising if it doesn&#39;t expand to a binary."},{"ref":"Cldr.Message.Backend.html#validate_bindings!/2","title":"Cldr.Message.Backend.validate_bindings!/2","type":"function","doc":""},{"ref":"Cldr.Message.ParseError.html","title":"Cldr.Message.ParseError","type":"exception","doc":"Exception raised when parsing an ICU message format"},{"ref":"Cldr.Message.Parser.html","title":"Cldr.Message.Parser","type":"module","doc":"Implements a parser for the ICU Message format"},{"ref":"Cldr.Message.Parser.html#message/2","title":"Cldr.Message.Parser.message/2","type":"function","doc":"Parses the given binary as message. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the message (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Cldr.Message.Parser.html#parse/2","title":"Cldr.Message.Parser.parse/2","type":"function","doc":""},{"ref":"Cldr.Message.Parser.html#plural_message/2","title":"Cldr.Message.Parser.plural_message/2","type":"function","doc":"Parses the given binary as plural_message. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the plural_message (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"readme.html","title":"Cldr Messages","type":"extras","doc":"Cldr Messages"},{"ref":"readme.html#introduction-and-getting-started","title":"Cldr Messages - Introduction and Getting Started","type":"extras","doc":"Implements the ICU Message Format for Elixir. In any application that addresses audiences from different cultures, the need arises to support the presentation of user interfaces, messages, alerts and other content in the appropriate language for a user. For nearly 30 years the go-to solution for this requirement in many computer langauges is gettext. There is a full-featured implementation for Elixir that is installed by default with Phoenix with over 10,000,000 downloads. Given the maturity and widespread adoption of Gettext, why implement another format? Leveraging the content from the Unicode CLDR project we can address some of the shortcomings of Gettext. A good description of motivations and differences can be found in this presentation by Mark Davis from Google in 2012. Two specific shortcomings that the ICU message format addresses: Grammatical Gender Many languages inflect in gender specific way. One example in French might be: # You are the only participant for a male and female Vous êtes the seul participant Vous êtes la seule participante # Married for a male and a female Marié Mariée In Gettext this requires individual messages and conditional code in the application in order to present the correct message to an audience. This is compounded by the fact that some languages have more than two grammatical genders (most have been two and four but but some are attested with up to 20. The ICU message format provides a mechanism (the choice format) that helps translator and UX designers implement a single message to easily encapsulate messages conditional on grammatical gender (or any other selector) Standardised plural rules Although Gettext supports pluralisation for messages through the Gettext.Plural module in Elixir and the Gettext functions like Gettext.ngettext/4, the plural rules for a language have to be implemented for each message. Give the wide differences in how plural forms are structured in different languages this can be a material challenge. For example: English has two plural forms: singular and plural French applies the singular rule to two values and a plural form to larger groupings Japanese does not differentiate Russian has 4 categories Arabic has 6 categories Since CLDR has a strong set of pluralization rules defined for ~500 locales, each of which is supported by ex_cldr for Elixir, the ICU message format can reuse these pluralization rules in a simple and consisten fashion using the [plural format]{#Plural_Format}"},{"ref":"readme.html#message-format-overview","title":"Cldr Messages - Message format overview","type":"extras","doc":"ICU message formats are Elixir strings with embedded formatting directives inserted between {}. Some examples: # Insert the binding `name` into the string &quot;My name is {name}&quot; # Insert a date, formatting in a localized `short` format plus a localized plural form # for the binding `num_photos` &quot;On {taken_date, date, short} {name} took {num_photos, plural, =0 {no photos.} =1 {one photo.} other {# photos.}}&quot; # Insert localized messages based upon the gender of the audience with # appropriate localized plural forms &quot;{gender_of_host, select, female { {num_guests, plural, offset: 1 =0 {{host} does not give a party.} =1 {{host} invites {guest} to her party.} =2 {{host} invites {guest} and one other person to her party.} other {{host} invites {guest} and # other people to her party.}}} male { {num_guests, plural, offset: 1 =0 {{host} does not give a party.} =1 {{host} invites {guest} to his party.} =2 {{host} invites {guest} and one other person to his party.} other {{host} invites {guest} and # other people to his party.}}} other { {num_guests, plural, offset: 1 =0 {{host} does not give a party.} =1 {{host} invites {guest} to their party.} =2 {{host} invites {guest} and one other person to their party.} other {{host} invites {guest} and # other people to their party.}}} }&quot;"},{"ref":"readme.html#message-formatting","title":"Cldr Messages - Message formatting","type":"extras","doc":"Using the above messages as examples: iex&gt; Cldr.Message.format! &quot;My name is {name}&quot;, name: &quot;Kip&quot; &quot;My name is Kip&quot; iex&gt; Cldr.Message.to_string! &quot;On {taken_date, date, short} {name} took {num_photos, plural, =0 {no photos.} =1 {one photo.} other {# photos.}}&quot;, taken_date: Date.utc_today, name: &quot;Kip&quot;, num_photos: 10 &quot;On 8/26/19 Kip took 10 photos.&quot; As of ex_cldr_messages version 0.3.0 a macro form is introduced which parses the message at compile time in order to optimize performance at run time. To use the macro, a backend module must be imported (or required) into a module that uses formatting. For example: defmodule SomeModule do # Import a &lt;backend&gt;.Cldr.Message module import MyApp.Cldr.Message def my_function do format(&quot;this is a string with a param {param}&quot;, param: 3) end end"},{"ref":"readme.html#installation","title":"Cldr Messages - Installation","type":"extras","doc":"def deps do [ {:ex_cldr_messages, &quot;~&gt; 0.3.0&quot;} ] end Documentation is at https://hexdocs.pm/cldr_messages."},{"ref":"readme.html#to-do","title":"Cldr Messages - To Do","type":"extras","doc":"For the initial release. This is a simple function interface to message formatting. Before 1.0 it needs to also have a means like gettext of managing messages in multiple different locales for the same message content. [X] Ignore whitespace between nested complex arguments at the top level. Example: {:select, {:named_arg, &quot;gender_of_host&quot;}, %{ &quot;female&quot; =&gt; [ {:literal, &quot;\\n &quot;}, &lt;---- Ignore this when its whitespace only {:plural, {:named_arg, &quot;num_guests&quot;}, [X] Support decimal numbers for selectors [X] Won&#39;t do. Support spellout format for Money.t types ? (Maybe can&#39;t because of floating point RBNF rule limitations) [X] Check for all occurences in README&#39;s for Cldr.get_current_locale/0 and change it to Cldr.get_locale/0 [X] Implement explicit =0 argument selection for plurals [X] Add remaining formatters for dates, times, datetimes [X] In ex_money, if no configured default_cldr_backend, delegate to Cldr.default_backend/0 [X] Implement a {arg, list, format} formatter that uses ex_cldr_lists [X] Implement a {arg, unit, format} formatter that uses ex_cldr_units [X] Implement offset [X] Implement custom formats in backend config provider; probably requires updating the struct in ex_cldr [X] Implement selectordinal [X] Assert that plural, select and selectordinal all have an other clause [X] Tests [X] @specs [ ] Dialyzer. Ask José to push nimble_parsec 0.5.2 to remove combinator errors [ ] Documentation [ ] Implement to_message for parse trees. This will define a canonical form which we can use to compare messages and create keys."},{"ref":"license.html","title":"LICENSE","type":"extras","doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS APPENDIX: How to apply the Apache License to your work. To apply the Apache License to your work, attach the following boilerplate notice, with the fields enclosed by brackets &quot;[]&quot; replaced with your own identifying information. (Don&#39;t include the brackets!) The text should be enclosed in the appropriate comment syntax for the file format. We also recommend that a file or class name and description of purpose be included on the same &quot;printed page&quot; as the copyright notice for easier identification within third-party archives. Copyright 2019 Kip Cole Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License."},{"ref":"changelog.html","title":"Changelog for Cldr_Messages v0.5.0","type":"extras","doc":"Changelog for Cldr_Messages v0.5.0 This is the changelog for CldrMessages v0.5.0 released on _, 2019. For older changelogs please consult the release tag on GitHub Enhancements Adds compile time checking that bindings are provided to the format/3 macro wherever possible Changelog for Cldr_Messages v0.4.0 This is the changelog for Cldr_Messages v0.4.0 released on August 29th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Conditionally compile functions that depend on optional dependencies Changelog for Cldr_Messages v0.3.0 This is the changelog for Cldr_Messages v0.3.0 released on August 29th, 2019. For older changelogs please consult the release tag on GitHub Breaking Changes Standardize on the Cldr.Message.format/3 as the public api. Cldr.Message.to_string/3 is removed. Enhancements Add the macro &lt;backend&gt;.Cldr.Message.format/3 to parse messages at compile time as a way to optiise performance at runtime. To use it add import &lt;backend&gt;.Cldr.Message to your module and use format/3. An example: defmodule SomeModule do import MyApp.Cldr.Message def my_function do format(&quot;this is a string with a param {param}&quot;, param: 3) end end Add Cldr.Mesasge.format_to_list/3 formats to an io_list Bug Fixes Fix dialyzer warnings. There are some warnings from combinators that will require nimble_parsec version 0.5.2 to be published before they are resolved. Changelog for Cldr_Messages v0.2.0 This is the changelog for Cldr_Messages v0.2.0 released on August 27th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Uses Cldr.Number.to_string/3 to format simple arguments that are numeric (integer, float and decimal). This gives a localised number format. An example: iex&gt; Cldr.Message.to_string &quot;You have {number} jelly beans&quot;, number: 1234 &quot;You have 1,234 jelly beans&quot; Similarly applies localized formatting for dates, times, datetimes. Changelog for Cldr_Messages v0.1.0 This is the changelog for Cldr_Messages v0.1.0 released on August 26th, 2019. For older changelogs please consult the release tag on GitHub Initial release. This release implements Cldr.Message.to_string/3 and Cldr.Message.format/3 This initial release is the basis for building a complete message localization solution as an alternative to Gettext. There is a long way to go until that is accomplished."}]